@@ -2,6 +2,8 @@
 
 import gtk
 import sets, time, os, glob
+import traceback
+import logging
 
 import IconManager
 import MediaManager
@@ -81,20 +83,20 @@ class DownloadView:
 		column.pack_start(renderer, True)
 		column.set_attributes(renderer, pixbuf=D_PIXBUF)
 		self._downloads_listview.append_column(column)
-		
-		column = gtk.TreeViewColumn(_('Description'))
-		column.set_resizable(True)
-		renderer = gtk.CellRendererText()
-		column.pack_start(renderer, True)
-		column.set_attributes(renderer, markup=D_DESCRIPTION_MARKUP)
-		self._downloads_listview.append_column(column)
-		
+
 		column = gtk.TreeViewColumn(_('Progress'))
 		column.set_resizable(True)
 		renderer = gtk.CellRendererProgress()
 		column.pack_start(renderer, True)
 		column.set_attributes(renderer, value=D_PROGRESS)
 		self._downloads_listview.append_column(column)
+				
+		column = gtk.TreeViewColumn(_('Description'))
+		column.set_resizable(True)
+		renderer = gtk.CellRendererText()
+		column.pack_start(renderer, True)
+		column.set_attributes(renderer, markup=D_DESCRIPTION_MARKUP)
+		self._downloads_listview.append_column(column)
 		
 		column = gtk.TreeViewColumn(_('Size'))
 		column.set_resizable(True)
@@ -128,7 +130,7 @@ class DownloadView:
 		removed   = list(oldset.difference(newset))
 		added     = list(newset.difference(oldset))
 		unchanged = list(oldset.intersection(newset))
-
+		
 		#slower but works better, because the list is changing all over the place
 		for item in removed:
 			i=-1
@@ -190,9 +192,13 @@ class DownloadView:
 
 		for media_id in added:
 			item        = self._downloads[current_list.index(media_id)]
-			entry       = self._db.get_entry(item.media['entry_id'])
-			description = self._db.get_feed_title(entry['feed_id']) + " &#8211; "+ entry['title']
-			size        = utils.format_size(item.total_size)
+			try:
+				entry       = self._db.get_entry(item.media['entry_id'])
+				description = self._db.get_feed_title(entry['feed_id']) + " &#8211; "+ entry['title']
+				size        = utils.format_size(item.total_size)
+			except:
+				logging.warning("trouble getting entry updating downloads: %s" % str(item))
+				continue
 			if item.status == PAUSED:
 				description_markup = '<span color="#777"><i>'+utils.my_quote(description)+'</i></span>'
 				size_markup = '<span color="#777"><i>'+size+'</i></span>'
@@ -206,7 +212,8 @@ class DownloadView:
 				size_markup = size
 				status_markup = ""
 
-			pixbuf     = self._get_pixbuf(entry['feed_id'])
+			pixbuf = self._icon_manager.get_icon_pixbuf(entry['feed_id'], 
+					   MAX_WIDTH, MAX_HEIGHT, MIN_SIZE, MIN_SIZE)
 			self._downloads_liststore.append([media_id, 
 											  description, 
 											  description_markup,
@@ -221,35 +228,6 @@ class DownloadView:
 		id_list = [row[D_MEDIA_ID] for row in self._downloads_liststore]
 		self._downloads.sort(lambda x,y: id_list.index(x.media['media_id']) - id_list.index(y.media['media_id']))
 			
-	def _get_pixbuf(self, feed_id):
-		filename = self._icon_manager.get_icon(feed_id)
-		if filename is None:
-			p = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB,True,8, MIN_SIZE, MIN_SIZE)
-			p.fill(0xffffff00)
-			return p
-	
-		try:
-			p = gtk.gdk.pixbuf_new_from_file(filename)
-		except:
-			p = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB,True,8, MIN_SIZE, MIN_SIZE)
-			p.fill(0xffffff00)
-			return p
-		height = p.get_height()
-		width = p.get_width()
-		if height > MAX_HEIGHT:
-			height = MAX_HEIGHT
-			width = p.get_width() * height / p.get_height()
-		if width > MAX_WIDTH:
-			width = MAX_WIDTH
-			height = p.get_height() * width / p.get_width()
-		if width < MIN_SIZE and height < MIN_SIZE:
-			height = MIN_SIZE
-			width = p.get_width() * height / p.get_height()
-		if height != p.get_height() or width != p.get_width():
-			del p
-			p = gtk.gdk.pixbuf_new_from_file_at_size(filename, width, height)
-		return p
-		
 	def on_stop_toolbutton_clicked(self, widget):
 		selection = self._downloads_listview.get_selection()
 		tree,selected = selection.get_selected_rows()
@@ -259,7 +237,6 @@ class DownloadView:
 		for medium in medialist:
 			print "stopping",medium['url']
 			self._app.do_cancel_download(medium)
-		self.update_downloads()
 		selection.unselect_all()
 		
 	def on_pause_toolbutton_clicked(self, widget):
@@ -267,16 +244,12 @@ class DownloadView:
 		tree,selected = selection.get_selected_rows()
 		for index in selected:
 			self._app.do_pause_download(self._downloads_liststore[index[0]][D_MEDIA_ID])
-		self.update_downloads()
-		selection.unselect_all()
 
 	def on_resume_toolbutton_clicked(self, widget):
 		selection = self._downloads_listview.get_selection()
 		tree,selected = selection.get_selected_rows()
 		for index in selected:
 			self._app.do_resume_download(self._downloads_liststore[index[0]][D_MEDIA_ID])
-		self.update_downloads()
-		selection.unselect_all()
 		
 	def on_download_list_row_activated(self, treeview, path, viewcolumn):
 		d = self._downloads[path[0]]
