@@ -4,6 +4,12 @@
 #Copyright 2006, Owen Williams
 #License: GPL
 
+import sys,os,os.path
+import pickle
+import urllib
+import getopt
+from math import ceil, floor
+
 import pygst
 pygst.require("0.10")
 import gst
@@ -16,18 +22,22 @@ import gobject
 
 import locale
 import gettext
+import logging
+logging.basicConfig(level=logging.DEBUG)
 
 locale.setlocale(locale.LC_ALL, '')
 _=gettext.gettext
 
-import sys,os,os.path
-import pickle
-import urllib
-
 if os.environ.has_key('SUGAR_PENGUINTV'):
 	RUNNING_SUGAR = True
 else:
 	RUNNING_SUGAR = False
+	
+try:
+	import hildon
+	RUNNING_HILDON = True
+except:
+	RUNNING_HILDON = False
 
 class GStreamerPlayer(gobject.GObject):
 
@@ -37,14 +47,27 @@ class GStreamerPlayer(gobject.GObject):
                            ([])),
 		'paused': (gobject.SIGNAL_RUN_FIRST, 
                            gobject.TYPE_NONE, 
-                           ([]))
+                           ([])),
+		'tick': (gobject.SIGNAL_RUN_FIRST, 
+                           gobject.TYPE_NONE, 
+                           ([])),
+        'item-queued': (gobject.SIGNAL_RUN_LAST, 
+        				gobject.TYPE_NONE, 
+        				[str, str, gobject.TYPE_PYOBJECT]),
+        'item-not-supported': (gobject.SIGNAL_RUN_LAST, 
+        					   gobject.TYPE_NONE, 
+        					   [str, str, gobject.TYPE_PYOBJECT]),
+        'items-removed': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, [])
 	}
 
-	def __init__(self, layout_dock):
+	def __init__(self, layout_dock, playlist, tick_interval=1):
 		gobject.GObject.__init__(self)
 		self._layout_dock = layout_dock
+		self._playlist_name = playlist
 		self._media_duration = 0
 		self._media_position = 0
+		self._last_tick = 0
+		self._tick_interval = tick_interval * gst.SECOND
 		self._last_index = -1
 		self._current_index = 0 #index to tree model
 		self._resized_pane = False
@@ -52,13 +75,11 @@ class GStreamerPlayer(gobject.GObject):
 		self.__is_exposed = False
 		self._x_overlay = None
 		self._prepare_save = False
+		self._do_stop_resume = False
+		self._has_video = False
 		
 		self._error_dialog = GStreamerErrorDialog()
 		
-		gobject.signal_new('item-queued', GStreamerPlayer, gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, [str, str, gobject.TYPE_PYOBJECT])
-		gobject.signal_new('item-not-supported', GStreamerPlayer, gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, [str, str, gobject.TYPE_PYOBJECT])
-		gobject.signal_new('items-removed', GStreamerPlayer, gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, [])
-		
 		gobject.timeout_add(300000, self._periodic_save_cb)
 		
 	###public functions###
@@ -105,86 +126,124 @@ class GStreamerPlayer(gobject.GObject):
 		column.set_attributes(renderer, markup=1)
 		self._queue_listview.append_column(column)
 		self._queue_listview.connect('row-activated', self._on_queue_row_activated)
+		self._queue_listview.connect('button-press-event', self._on_queue_row_button_press)
 		self._queue_listview.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
 		#dnd reorder
 		self._TARGET_TYPE_REORDER = 80
-		drop_types = [('reorder',gtk.TARGET_SAME_WIDGET, self._TARGET_TYPE_REORDER)]
+		self._TARGET_TYPE_URI_LIST = 81
+		drop_types = [('reorder',gtk.TARGET_SAME_WIDGET, self._TARGET_TYPE_REORDER),
+					  ('text/uri-list',0,self._TARGET_TYPE_URI_LIST)]
 		#for removing items from favorites and reordering
 		self._queue_listview.enable_model_drag_source(gtk.gdk.BUTTON1_MASK, drop_types, gtk.gdk.ACTION_MOVE)
-		self._queue_listview.enable_model_drag_dest(drop_types, gtk.gdk.ACTION_MOVE)
+		self._queue_listview.enable_model_drag_dest(drop_types, gtk.gdk.ACTION_DEFAULT)
 		self._queue_listview.connect('drag-data-received', self._on_queue_drag_data_received)
 		
 		s_w.add(self._queue_listview)
 		self._sidepane_vbox.pack_start(s_w, True)
-		button_box = gtk.HButtonBox()
-		button_box.set_property('layout-style', gtk.BUTTONBOX_END)
+		
 		button = gtk.Button(stock='gtk-remove')
 		button.connect("clicked", self._on_remove_clicked)
-		button_box.add(button)
-		self._sidepane_vbox.pack_start(button_box, False)
+		if not RUNNING_HILDON:
+			button_box = gtk.HButtonBox()
+			button_box.set_property('layout-style', gtk.BUTTONBOX_END)
+			button_box.add(button)
+			self._sidepane_vbox.pack_start(button_box, False)
+		else:
+			list_bottom_hbox = gtk.HBox(False)
+			list_bottom_hbox.pack_start(button, False)
+			self._sidepane_vbox.pack_start(list_bottom_hbox, False)
 		
 		self._hpaned.add2(self._sidepane_vbox)
 		
 		main_vbox.add(self._hpaned)
 		
-		
 		self._controls_hbox = gtk.HBox()
 		self._controls_hbox.set_spacing(6)
-		button_box = gtk.HButtonBox()
-		button_box.set_property('layout-style', gtk.BUTTONBOX_START)
 		
-		image = gtk.Image()
-		image.set_from_stock("gtk-media-previous",gtk.ICON_SIZE_BUTTON)
-		button = gtk.Button("")
-		button.set_image(image)
+		button_box = gtk.HButtonBox()
+		button_box.set_homogeneous(False)
+		button_box.set_property('layout-style', gtk.BUTTONBOX_START)
+
+		if not RUNNING_HILDON:
+			image = gtk.Image()
+			image.set_from_stock("gtk-media-previous",gtk.ICON_SIZE_BUTTON)
+			button = gtk.Button("")
+			button.set_image(image)
+		else:
+			button = gtk.Button()
+			label = gtk.Label(_("Prev"))
+			label.set_use_markup(True)
+			button.add(label)
+			label.show()
 		button.set_property('can-focus', False)
 		button.connect("clicked", self._on_prev_clicked)
-		button_box.add(button)
+		button_box.pack_start(button, True, True)
 		
-		image = gtk.Image()
-		image.set_from_stock("gtk-media-rewind",gtk.ICON_SIZE_BUTTON)
-		button = gtk.Button("")
-		button.set_image(image)
+		if not RUNNING_HILDON:
+			image = gtk.Image()
+			image.set_from_stock("gtk-media-rewind",gtk.ICON_SIZE_BUTTON)
+			button = gtk.Button("")
+			button.set_image(image)
+		else:
+			button = gtk.Button(_("Rew"))
 		button.set_property('can-focus', False)
 		button.connect("clicked", self._on_rewind_clicked)
-		button_box.add(button)
+		button_box.pack_start(button, True, True)
 		
-		image = gtk.Image()
-		image.set_from_stock("gtk-media-play",gtk.ICON_SIZE_BUTTON)
-		self._play_pause_button = gtk.Button("")
-		self._play_pause_button.set_image(image)
+		if not RUNNING_HILDON:
+			image = gtk.Image()
+			image.set_from_stock("gtk-media-play",gtk.ICON_SIZE_BUTTON)
+			self._play_pause_button = gtk.Button("")
+			self._play_pause_button.set_image(image)
+		else:
+			self._play_pause_button = gtk.Button(_("Play"))
 		self._play_pause_button.set_property('can-focus', False)
 		self._play_pause_button.connect("clicked", self._on_play_pause_toggle_clicked)
-		button_box.add(self._play_pause_button)
+		button_box.pack_start(self._play_pause_button, True, True)
 	
-		image = gtk.Image()
-		image.set_from_stock("gtk-media-stop",gtk.ICON_SIZE_BUTTON)
-		button = gtk.Button("")
-		button.set_image(image)
+		if not RUNNING_HILDON:
+			image = gtk.Image()
+			image.set_from_stock("gtk-media-stop",gtk.ICON_SIZE_BUTTON)
+			button = gtk.Button("")
+			button.set_image(image)
+		else:
+			button = gtk.Button(_("Stop"))
 		button.set_property('can-focus', False)
 		button.connect("clicked", self._on_stop_clicked)
-		button_box.add(button)
+		button_box.pack_start(button, True, True)
 		
-		image = gtk.Image()
-		image.set_from_stock("gtk-media-forward",gtk.ICON_SIZE_BUTTON)
-		button = gtk.Button("")
-		button.set_image(image)
+		if not RUNNING_HILDON:
+			image = gtk.Image()
+			image.set_from_stock("gtk-media-forward",gtk.ICON_SIZE_BUTTON)
+			button = gtk.Button("")
+			button.set_image(image)
+		else:
+			button = gtk.Button(_("FF"))
 		button.set_property('can-focus', False)
 		button.connect("clicked", self._on_forward_clicked)
-		button_box.add(button)
+		button_box.pack_start(button, True, True)
 		
-		image = gtk.Image()
-		image.set_from_stock("gtk-media-next",gtk.ICON_SIZE_BUTTON)
-		button = gtk.Button("")
-		button.set_image(image)
+		if not RUNNING_HILDON:
+			image = gtk.Image()
+			image.set_from_stock("gtk-media-next",gtk.ICON_SIZE_BUTTON)
+			button = gtk.Button("")
+			button.set_image(image)
+		else:
+			button = gtk.Button(_("Next"))
 		button.set_property('can-focus', False)
 		button.connect("clicked", self._on_next_clicked)
-		button_box.add(button)
+		button_box.pack_start(button, True, True)
 		
 		self._controls_hbox.pack_start(button_box, False)
+		
 		self._time_label = gtk.Label("")
 		self._time_label.set_alignment(0.0,0.5)
-		self._controls_hbox.pack_start(self._time_label, True)
+		
+		if not RUNNING_HILDON:
+			self._controls_hbox.pack_start(self._time_label, True)
+		else:
+			list_bottom_hbox.pack_start(self._time_label, True)
+			list_bottom_hbox.reorder_child(self._time_label, 0)
 		
 		main_vbox.pack_start(self._controls_hbox, False)
 		self._layout_dock.add(main_vbox)
@@ -194,9 +253,14 @@ class GStreamerPlayer(gobject.GObject):
 		#use default audio sink, but get our own video sink
 		self._v_sink = self._get_video_sink()
 		self._pipeline.set_property('video-sink',self._v_sink)
+		#if RUNNING_HILDON:
+		#	self._mp3_sink = gst.element_factory_make('dspmp3sink', 'mp3sink')
+		#	self._pcm_sink = gst.element_factory_make('dsppcmsink', 'pcmsink')
+			
 		bus = self._pipeline.get_bus()
 		bus.add_signal_watch()
 		bus.connect('message', self._on_gst_message)
+		#bus.connect('sync-message::element', self._on_sync_message)
 
 		self._layout_dock.show_all()
 		
@@ -206,6 +270,9 @@ class GStreamerPlayer(gobject.GObject):
 	def is_exposed(self):
 		return self.__is_exposed
 		
+	def has_video(self):
+		return self._has_video
+		
 	def detach(self):
 		"""video window can detach.  queue stays embedded"""
 		pass
@@ -225,13 +292,8 @@ class GStreamerPlayer(gobject.GObject):
 			self._sidepane_vbox.hide()
 			
 	def load(self):
-		if RUNNING_SUGAR:
-			import sugar.env
-			home = os.path.join(sugar.env.get_profile_path(), 'penguintv')
-		else:
-			home = os.path.join(os.getenv('HOME'), ".penguintv")
 		try:
-			playlist = open(os.path.join(home, 'gst_playlist.pickle'), 'r')
+			playlist = open(self._playlist_name, 'r')
 		except:
 			print "error reading playlist"
 			return
@@ -252,13 +314,8 @@ class GStreamerPlayer(gobject.GObject):
 		
 	def save(self):
 		"""saves playlist"""
-		if RUNNING_SUGAR:
-			import sugar.env
-			home = os.path.join(sugar.env.get_profile_path(), 'penguintv')
-		else:
-			home = os.path.join(os.getenv('HOME'), ".penguintv")
 		try:
-			playlist = open(os.path.join(home, 'gst_playlist.pickle'), 'w')
+			playlist = open(self._playlist_name, 'w')
 		except:
 			print "error writing playlist"
 			return
@@ -282,10 +339,67 @@ class GStreamerPlayer(gobject.GObject):
 		if name is None:
 			name = os.path.split(filename)[1]
 		
-		#thanks gstfile.py
-		self.current = Discoverer(filename)
-		self.current.connect('discovered', self._on_type_discovered, filename, name, userdata)
-		self.current.discover()	
+		if RUNNING_HILDON:
+			ext = os.path.splitext(filename)[1][1:]
+			known_good = ['mp3', 'wav', 'm4a', 'wma', 'mpg', 'avi', '3gp', 'rm', 'asf', 'mp4']
+			try:
+				gst.element_factory_make("oggdemux", "test")
+				known_good += ['ogg']
+			except:
+				pass
+				
+			self._on_type_discovered(None, ext in known_good, filename, name, userdata)
+		else:
+			#thanks gstfile.py
+			d = Discoverer(filename)
+			d.connect('discovered', self._on_type_discovered, filename, name, userdata)
+			d.discover()
+			
+	def unqueue(self, filename=None, userdata=None):
+		model = self._queue_listview.get_model()
+		
+		iter_list = []
+		
+		if filename is not None:
+			logging.warning("UNTESTED CODE:")
+			it = model.get_iter_first()
+			while it is not None:
+				data = model.get(it, 0)[0]
+				logging.debug("%s %s" % (str(filename), str(data)))
+				if data == "file://" + filename:
+					iter_list.append(it)
+				it = model.iter_next(it)
+				
+			if len(iter_list) > 0:
+				self._remove_items(iter_list)
+				
+		if userdata is not None:
+			it = model.get_iter_first()
+			while it is not None:
+				data = model.get(it, 3)[0]
+				if data == userdata:
+					iter_list.append(it)
+				it = model.iter_next(it)
+				
+			if len(iter_list) > 0:
+				self._remove_items(iter_list)
+				
+	def relocate_media(self, old_dir, new_dir):
+		if old_dir[-1] == '/' or old_dir[-1] == '\\':
+			old_dir = old_dir[:-1]
+			
+		if new_dir[-1] == '/' or new_dir[-1] == '\\':
+			new_dir = new_dir[:-1]
+	
+		model = self._queue_listview.get_model()
+		if len(model) == 0:
+			return
+			
+		self.stop()
+			
+		for row in model:
+			if row[0].startswith("file://" + old_dir):
+				row[0] = row[0].replace(old_dir, new_dir)
 			
 	def get_queue_count(self):
 		return len(self._queue_listview.get_model())
@@ -319,10 +433,18 @@ class GStreamerPlayer(gobject.GObject):
 				return
 			self._prepare_display()
 			self._prepare_save = True
+		else:
+			if self._do_stop_resume:
+				self._do_stop_resume = False
+				self._prepare_display()
+				self._seek_to_saved_position()
 		self._pipeline.set_state(gst.STATE_PLAYING)
-		image = gtk.Image()
-		image.set_from_stock("gtk-media-pause",gtk.ICON_SIZE_BUTTON)
-		self._play_pause_button.set_image(image)
+		if not RUNNING_HILDON:
+			image = gtk.Image()
+			image.set_from_stock("gtk-media-pause",gtk.ICON_SIZE_BUTTON)
+			self._play_pause_button.set_image(image)
+		else:
+			self._play_pause_button.set_label(_("Pause"))
 		self._media_duration = -1
 		if not notick:
 			gobject.timeout_add(500, self._tick)
@@ -333,9 +455,12 @@ class GStreamerPlayer(gobject.GObject):
 		try: self._media_position = self._pipeline.query_position(gst.FORMAT_TIME)[0]
 		except: pass
 		self._pipeline.set_state(gst.STATE_PAUSED)
-		image = gtk.Image()
-		image.set_from_stock("gtk-media-play",gtk.ICON_SIZE_BUTTON)
-		self._play_pause_button.set_image(image)
+		if not RUNNING_HILDON:
+			image = gtk.Image()
+			image.set_from_stock("gtk-media-play",gtk.ICON_SIZE_BUTTON)
+			self._play_pause_button.set_image(image)
+		else:
+			self._play_pause_button.set_label(_("Play"))
 		self.emit('paused')
 		
 	def stop(self):
@@ -346,28 +471,38 @@ class GStreamerPlayer(gobject.GObject):
 		try: self._media_position = self._pipeline.query_position(gst.FORMAT_TIME)[0]
 		except: pass
 		self._pipeline.set_state(gst.STATE_READY)
-		self._last_index = -1
+		#self._last_index = -1
 		self._seek_scale.set_range(0,1)
 		self._seek_scale.set_value(0)
-		image = gtk.Image()
-		image.set_from_stock("gtk-media-play",gtk.ICON_SIZE_BUTTON)
-		self._play_pause_button.set_image(image)
+		self._do_stop_resume = True
+		if not RUNNING_HILDON:
+			image = gtk.Image()
+			image.set_from_stock("gtk-media-play",gtk.ICON_SIZE_BUTTON)
+			self._play_pause_button.set_image(image)
+		else:
+			self._play_pause_button.set_label(_("Play"))
+	
 		if 'gstxvimagesink' in str(type(self._v_sink)).lower():
 			#release the xv port
 			self._pipeline.unlink(self._v_sink)
 			self._v_sink.set_state(gst.STATE_NULL)
-			self._v_sink = self._get_video_sink(True)
-			self._pipeline.set_property('video-sink',self._v_sink)
+			if not RUNNING_HILDON:
+				self._v_sink = self._get_video_sink(True)
+				self._pipeline.set_property('video-sink',self._v_sink)
 		self.emit('paused')
 			
 	def ff(self):
-		new_pos = self._media_position+15000000000L #15 seconds I think
+		if self._pipeline.get_state()[1] != gst.STATE_PLAYING:
+			return
+		new_pos = self._media_position + 15 * gst.SECOND
 		if new_pos > self._media_duration:
 			new_pos = self._media_duration
 		self.seek(new_pos)
 		
 	def rew(self):
-		new_pos = self._media_position-5000000000L #5 seconds I think
+		if self._pipeline.get_state()[1] != gst.STATE_PLAYING:
+			return
+		new_pos = self._media_position - 5 * gst.SECOND
 		if new_pos < 0:
 			new_pos = 0
 		self.seek(new_pos)
@@ -385,6 +520,7 @@ class GStreamerPlayer(gobject.GObject):
 		
 		self._seek_scale.set_range(0,1)
 		self._seek_scale.set_value(0)
+		self._do_stop_resume = False
 		self.play()
 		
 	def prev(self):
@@ -398,6 +534,7 @@ class GStreamerPlayer(gobject.GObject):
 		selection.select_path((self._current_index,))
 		self._seek_scale.set_range(0,1)
 		self._seek_scale.set_value(0)
+		self._do_stop_resume = False
 		self.play()
 		
 	def finish(self):
@@ -410,6 +547,16 @@ class GStreamerPlayer(gobject.GObject):
 							gst.SEEK_TYPE_SET, time,
 							gst.SEEK_TYPE_NONE, 0)
 							
+	def vol_up(self):
+		old_vol = floor(self._pipeline.get_property('volume'))
+		if old_vol < 10:
+			self._pipeline.set_property('volume', old_vol + 1)
+		
+	def vol_down(self):
+		old_vol = ceil(self._pipeline.get_property('volume'))
+		if old_vol > 0: 
+			self._pipeline.set_property('volume', old_vol - 1)
+							
 	###handlers###
 		
 	def _on_play_clicked(self, b): self.play()
@@ -432,7 +579,7 @@ class GStreamerPlayer(gobject.GObject):
 		if ismedia:
 			model = self._queue_listview.get_model()
 			uri = 'file://'+urllib.quote(filename)
-			if RUNNING_SUGAR:
+			if RUNNING_SUGAR or RUNNING_HILDON:
 				name = '<span size="x-small">'+name+'</span>'
 			model.append([uri, name, "", userdata])
 			self.emit('item-queued', filename, name, userdata)
@@ -442,16 +589,16 @@ class GStreamerPlayer(gobject.GObject):
 	
 	def _on_remove_clicked(self, b):
 		model, paths = self._queue_listview.get_selection().get_selected_rows()
-		
+		self._remove_items([model.get_iter(path) for path in paths])
+			
+	def _remove_items(self, iter_list):
+		model = self._queue_listview.get_model()
 		current_uri = model[self._current_index][0]
 		
-		iter_list = []
-		for path in paths:
-			if path[0] == self._current_index:
-				self.stop()
-			iter_list.append(model.get_iter(path))
-			
 		for i in iter_list:
+			if model.get_path(i)[0] == self._current_index:
+				print "stopping current"
+				self.stop()
 			model.remove(i)
 			
 		if len(model) == 0:
@@ -485,29 +632,49 @@ class GStreamerPlayer(gobject.GObject):
 		self._current_index = path[0]
 		self.play()
 		
+	def _on_queue_row_button_press(self, widget, event):
+		if event.button==3: #right click     
+			menu = gtk.Menu()   
+			
+			path = widget.get_path_at_pos(int(event.x),int(event.y))
+			model = widget.get_model()
+			if path is None: #nothing selected
+				return
+
+			item = gtk.ImageMenuItem(_("_Remove"))
+			img = gtk.image_new_from_stock('gtk-remove',gtk.ICON_SIZE_MENU)
+			item.set_image(img)
+			item.connect('activate',self._on_remove_clicked)
+			menu.append(item)
+				
+			menu.show_all()
+			menu.popup(None,None,None, event.button,event.time)
+			
 	def _on_key_press_event(self, widget, event):
 		keyname = gtk.gdk.keyval_name(event.keyval)
+		self.handle_key(keyname)
+	
+	def handle_key(self, keyname):
 		#if keyname == 'f':
 		#	self.toggle_fullscreen()
 		if keyname == 'n':
 			self.next()
-		elif keyname == 'p':
+		elif keyname == 'b':
 			self.prev()
-		elif keyname == 'space':
+		elif keyname == 'space' or keyname == 'p':
 			self.play_pause_toggle()
 		#FIXME: these don't work when we're embedded in penguintv.  why?
 		elif keyname == 'Right':
 			self.ff()
 		elif keyname == 'Left':
 			self.rew()
-		#else:
-		#	print keyname
+		else:
+			return False
+		return True
 			
 	def _on_drawing_area_exposed(self, widget, event):
 		if self._x_overlay is None:
 			self._x_overlay = self._pipeline.get_by_interface(gst.interfaces.XOverlay)
-		#if self._x_overlay is not None:
-		#	self._x_overlay.expose()
 		self._v_sink.expose()
 		if not self.__is_exposed:
 			model = self._queue_listview.get_model()
@@ -518,6 +685,22 @@ class GStreamerPlayer(gobject.GObject):
 				
 	###utility functions###
 	def _get_video_sink(self, compat=False):
+		if RUNNING_HILDON:
+			if compat:
+				return none
+			try:
+				v_sink = self._pipeline.get_by_name('videosink')
+				if v_sink is None:
+					v_sink = gst.element_factory_make("xvimagesink", "v_sink")
+					bus = self._pipeline.get_bus()
+					v_sink.set_bus(bus)
+					return v_sink
+				else:
+					return v_sink
+			except:
+				logging.error("didn't get videosink :(")
+				return None
+				
 		if compat:
 			sinks = ["ximagesink"]
 		else:
@@ -535,6 +718,14 @@ class GStreamerPlayer(gobject.GObject):
 
 	def _ready_new_uri(self, uri):
 		"""load a new uri into the pipeline and prepare the pipeline for playing"""
+		#if RUNNING_HILDON:
+		#	ext = os.path.splitext(uri)[1]
+		#	#if ext == '.mp3':
+		#	#	logging.debug("readying for mp3")
+		#	#	self._pipeline.set_property('audio-sink', self._mp3_sink)
+		#	#else:
+		#	#	logging.debug("readying for pcm")
+		#	#	self._pipeline.set_property('audio-sink', self._pcm_sink)
 		self._pipeline.set_state(gst.STATE_READY)
 		self._pipeline.set_property('uri',uri)
 		self._x_overlay = None #reset so we grab again when we start playing
@@ -544,6 +735,7 @@ class GStreamerPlayer(gobject.GObject):
 		#if type(self._v_sink) != GstXVImageSink and not compat:
 		#do this right at some point: if we are using a substandard sink
 		#and we're not being specifically told to use it, try the better one
+		
 		if 'gstximagesink' in str(type(self._v_sink)).lower() and not compat:
 			self._v_sink = self._get_video_sink()
 			self._pipeline.set_property('video-sink',self._v_sink)
@@ -552,7 +744,6 @@ class GStreamerPlayer(gobject.GObject):
 			self._pipeline.set_property('video-sink',self._v_sink)
 			
 		self._v_sink.set_state(gst.STATE_READY)	
-		#	
 		change_return, state, pending = self._v_sink.get_state(gst.SECOND * 10)
 		if change_return != gst.STATE_CHANGE_SUCCESS:
 			if 'gstximagesink' in str(type(self._v_sink)).lower():
@@ -563,16 +754,29 @@ class GStreamerPlayer(gobject.GObject):
 			self._pipeline.set_property('video-sink',self._v_sink)
 			self._prepare_display(True)
 			return
+ 		#maemo throws X Window System errors when doing this -- ignore them
+		#http://labs.morpheuz.eng.br/blog/14/08/2007/xv-and-mplayer-on-maemo/
+		if RUNNING_HILDON:
+			gtk.gdk.error_trap_push()
 				
 		self._v_sink.set_xwindow_id(self._drawing_area.window.xid)
+		#causes expose problems
+		#self._v_sink.set_property('sync', True)
 		self._v_sink.set_property('force-aspect-ratio', True)
 		self._resized_pane = False
 		
+  		if RUNNING_HILDON:
+  			def pop_trap():
+				gtk.gdk.flush()
+				gtk.gdk.error_trap_pop()
+				return False
+				
+			gobject.idle_add(pop_trap)
+		
 	def _resize_pane(self):
 		#get video width and height so we can resize the pane
 		#see totem
 		#if (!(caps = gst_pad_get_negotiated_caps (pad)))
-		
 		#unbreakme: if there's no video, it doesn't draw right here either
 		self._drawing_area.set_flags(gtk.DOUBLE_BUFFERED)
 		
@@ -581,7 +785,7 @@ class GStreamerPlayer(gobject.GObject):
 		
 		pad = self._v_sink.get_pad('sink')
 		if pad is None:
-			print "didn't get pad for resize info"
+			logging.debug("didn't get video sink pad")
 			return
 		
 		self._resized_pane = True	
@@ -589,8 +793,16 @@ class GStreamerPlayer(gobject.GObject):
  		caps = pad.get_negotiated_caps()
  		if caps is None: #no big deal, this might be audio only
  			self._hpaned.set_position(max_width / 2)
+ 			self._has_video = False
  			return
  		
+ 		#maemo throws X Window System errors when doing this -- ignore them
+		#http://labs.morpheuz.eng.br/blog/14/08/2007/xv-and-mplayer-on-maemo/
+		if RUNNING_HILDON:
+			gtk.gdk.error_trap_push()
+
+ 		self._has_video = True
+ 		
  		#unbreakme: without this option the video doesn't redraw correctly when exposed
 		self._drawing_area.unset_flags(gtk.DOUBLE_BUFFERED)	
  			
@@ -606,6 +818,14 @@ class GStreamerPlayer(gobject.GObject):
   		else:
   			self._hpaned.set_position(int(new_display_width))
   		
+  		if RUNNING_HILDON:
+			def pop_trap():
+				gtk.gdk.flush()
+				gtk.gdk.error_trap_pop()
+				return False
+				
+			gobject.idle_add(pop_trap)
+
 	def _seek_to_saved_position(self):
 		"""many sources don't support seek in ready, so we do it the old fashioned way:
 		play, wait for it to play, pause, wait for it to pause, and then seek"""
@@ -617,9 +837,15 @@ class GStreamerPlayer(gobject.GObject):
 			else: row[2] = ""
 		#save, because they may get overwritten when we play and pause
 		pos, dur = self._media_position, self._media_duration
-		volume = self._pipeline.get_property('volume')
+		if not RUNNING_HILDON:
+			volume = self._pipeline.get_property('volume')
 		#temporarily mute to avoid little bloops during this hack
 		self._pipeline.set_property('volume',0)
+		
+		#maemo throws X Window System errors when doing this -- ignore them
+		#http://labs.morpheuz.eng.br/blog/14/08/2007/xv-and-mplayer-on-maemo/
+		if RUNNING_HILDON:
+			gtk.gdk.error_trap_push()
 
 		self.play(True)
 		change_return, state, pending = self._pipeline.get_state(gst.SECOND * 10)
@@ -635,11 +861,30 @@ class GStreamerPlayer(gobject.GObject):
 		self._resize_pane()
 		self._seek_scale.set_range(0,self._media_duration)
 		self._seek_scale.set_value(self._media_position)
-		self._pipeline.set_property('volume',volume)
+		if not RUNNING_HILDON:
+			self._pipeline.set_property('volume',volume)
+		else:
+			self._pipeline.set_property('volume', 10)
 		self._update_time_label()
 		
+  		if RUNNING_HILDON:
+			def pop_trap():
+				gtk.gdk.flush()
+				gtk.gdk.error_trap_pop()
+				return False
+				
+			gobject.idle_add(pop_trap)
+		
 	def _tick(self):
 		self.__no_seek = True
+		try:
+			now = self._pipeline.query_position(gst.FORMAT_TIME)[0]
+			if now - self._last_tick > self._tick_interval:
+				self._last_tick = now
+				self.emit('tick')
+		except:
+			pass
+		
 		self._update_seek_bar()
 		self._update_time_label()
 		if self._prepare_save:
@@ -653,9 +898,12 @@ class GStreamerPlayer(gobject.GObject):
 		return True
 
 	def _update_seek_bar(self):
+		#for some reason when paused, hildon tells us the position is 0. Ignore it
+		if RUNNING_HILDON and self._pipeline.get_state()[1] == gst.STATE_PAUSED:
+			return
 		try:
 			self._media_position = self._pipeline.query_position(gst.FORMAT_TIME)[0]
-			#print self._media_duration
+			#print self._media_position, self._media_duration
 			if self._media_position > self._media_duration:
 				self._media_duration = self._pipeline.query_duration(gst.FORMAT_TIME)[0]
 				self._seek_scale.set_range(0,self._media_duration)
@@ -666,7 +914,7 @@ class GStreamerPlayer(gobject.GObject):
 			
 	def _update_time_label(self):
 		def nano_to_string(long_val):
-			seconds = long_val/1000000000
+			seconds = long_val / gst.SECOND
 			minutes = seconds / 60
 			seconds = seconds % 60
 			return "%i:%.2i" % (minutes,seconds)
@@ -688,14 +936,21 @@ class GStreamerPlayer(gobject.GObject):
 				self.stop()
 		elif message.type == gst.MESSAGE_ERROR:
 			gerror, debug = message.parse_error()
-			print "GSTREAMER ERROR:",debug
-			self._error_dialog.show_error(debug)
-			
+			logging.error("GSTREAMER ERROR: %s" % debug)
+			if not RUNNING_HILDON:
+				self._error_dialog.show_error(debug)
+		
+	#def _on_sync_message(self, bus, message):
+	#	if message.structure is None:
+	#		return
+	#	if message.structure.get_name() == 'prepare-xwindow-id':
+	#		logging.debug("forcing aspect to true")
+	#		message.src.set_property('force-aspect-ratio', True)
 	###drag and drop###
 		
 	def _on_queue_drag_data_received(self, treeview, context, x, y, selection, targetType, time):
-		treeview.emit_stop_by_name('drag-data-received')
 		if targetType == self._TARGET_TYPE_REORDER:
+			treeview.emit_stop_by_name('drag-data-received')
 			model, paths_to_copy = treeview.get_selection().get_selected_rows()
 			if len(paths_to_copy) > 1:
 				print "can only move one at a time"
@@ -724,6 +979,12 @@ class GStreamerPlayer(gobject.GObject):
 			except:
 				model.append(row)
 				context.finish(True, True, time)
+		elif targetType == self._TARGET_TYPE_URI_LIST:
+			uri_list = [s for s in selection.data.split('\r\n') if len(s) > 0]
+			for uri in uri_list:
+				uri = uri.replace("file://", "")
+				uri = urllib.unquote(uri)
+				self.queue_file(uri)			
 
 	def checkSanity(self, model, iter_to_copy, target_iter):
 		path_of_iter_to_copy = model.get_path(iter_to_copy)
@@ -781,32 +1042,63 @@ def do_quit(self, widget, player):
 def items_removed(player):
 	print player.get_queue_count()
 	
-def item_not_supported(player, filename, name):
+def item_not_supported(app, player, filename, name):
 	print filename,name, "not supported"
 	
 fullscreen = False
 def on_app_key_press_event(widget, event, player, window):
 	global fullscreen
 	keyname = gtk.gdk.keyval_name(event.keyval)
-	if keyname == 'f':
+	if keyname == 'f' or (RUNNING_HILDON and keyname == 'F6'):
+		#maemo throws X Window System errors when doing this -- ignore them
+		#http://labs.morpheuz.eng.br/blog/14/08/2007/xv-and-mplayer-on-maemo/
+		if RUNNING_HILDON:
+			gtk.gdk.error_trap_push()
 		fullscreen = not fullscreen
 		player.toggle_controls(fullscreen)
 		if fullscreen:
 			window.window.fullscreen()
 		else:
 			window.window.unfullscreen()
+		if RUNNING_HILDON:
+			def pop_trap():
+				gtk.gdk.flush()
+				gtk.gdk.error_trap_pop()
+				return False
+				
+			gobject.idle_add(pop_trap)
 		
 if __name__ == '__main__': # Here starts the dynamic part of the program 
 	window = gtk.Window()
-	app = GStreamerPlayer(window)
+	if RUNNING_SUGAR:
+		import sugar.env
+		home = os.path.join(sugar.env.get_profile_path(), 'penguintv')
+	else:
+		home = os.path.join(os.getenv('HOME'), ".penguintv")
+	try:
+		opts, args = getopt.getopt(sys.argv[1:], "p:", ["--playlist"])
+	except getopt.GetoptError, e:
+		print "error %s", str(e)
+		sys.exit(1)
+	playlist = os.path.join(home, 'gst_playlist.pickle')
+	if len(opts) > 0:
+		for o, a in opts:
+			if o in ('-p', '--playlist'):
+				playlist = a
+								
+	app = GStreamerPlayer(window, playlist)
 	app.Show()
-	app.load()
+	
 	window.connect('delete-event', do_quit, app)
 	window.connect('key-press-event', on_app_key_press_event, app, window)
 	window.resize(640,480)
 	app.connect('items-removed', items_removed)	
 	app.connect('item-not-supported', item_not_supported)
-	for item in sys.argv[1:]:
+	
+				
+	app.load()
+
+	for item in args:
 		app.queue_file(item)
 	gtk.main()
 	
