@@ -1,17 +1,35 @@
+# -*- coding: utf-8 -*-
 # Written by Owen Williams
 # see LICENSE for license information
+
 import os,sys,time, pwd, os.path
 import subprocess
 import string
-import fnmatch
 import urllib
 import HTMLParser
 import string
 import locale
 import gettext
 import shutil
+import logging
 
-import gtk
+try:
+	from sqlite3 import OperationalError as OperationalError
+except:
+	from pysqlite2.dbapi2 import OperationalError as OperationalError
+
+try:
+	import gtk
+	GTK_OK = True
+except:
+	GTK_OK = False
+	
+try:
+	import hildon
+	logging.debug("Detected hildon environment")
+	RUNNING_HILDON = True
+except:
+	RUNNING_HILDON = False
 
 from subprocess import Popen, PIPE, STDOUT
 
@@ -27,6 +45,8 @@ if RUNNING_SUGAR:
 	#I do this in case we're running in a python environment that has lucene
 	#and/or gconf but we want to pretend they aren't there
 	HAS_LUCENE = False
+	HAS_SEARCH = False
+	HAS_XAPIAN = False
 	HAS_GCONF = False
 	HAS_GNOMEVFS = False
 	HAS_PYXML = False
@@ -48,9 +68,18 @@ else:
 	except:
 		HAS_MOZILLA = False
 
+	HAS_SEARCH = False
+
+	try:
+		import xapian
+		HAS_XAPIAN = True
+		HAS_SEARCH = True
+	except:
+		HAS_XAPIAN = False
 	try:
 		import PyLucene
 		HAS_LUCENE = True
+		HAS_SEARCH = True
 	except:
 		HAS_LUCENE = False
 		
@@ -58,24 +87,37 @@ else:
 		import gconf
 		HAS_GCONF = True
 	except:
-		HAS_GCONF = False
+		try:
+			from gnome import gconf
+			HAS_GCONF = True
+		except:
+			HAS_GCONF = False
 		
 	try:
 		import gnomevfs
 		HAS_GNOMEVFS = True
 	except:
-		HAS_GNOMEVFS = False
+		try:
+			from gnome import gnomevfs
+			HAS_GNOMEVFS = True
+		except:
+			HAS_GNOMEVFS = False
 		
 	try:
-		from xml.sax.saxutils import DefaultHandler
+		from xml.sax.handler import ContentHandler
 		HAS_PYXML = True
 	except:
-		HAS_PYXML = False
+		try:
+			from xml.sax.saxutils import DefaultHandler
+			HAS_PYXML = True
+		except:
+			HAS_PYXML = False
 		
-	if gtk.pygtk_version >= (2, 10, 0):
-		HAS_STATUS_ICON = True
-	else:
-		HAS_STATUS_ICON = False
+	if GTK_OK:
+		if gtk.pygtk_version >= (2, 10, 0):
+			HAS_STATUS_ICON = True
+		else:
+			HAS_STATUS_ICON = False
 		
 	try:
 		import pynotify
@@ -90,12 +132,28 @@ else:
 		HAS_GSTREAMER = True
 	except:
 		HAS_GSTREAMER = False
+
+if RUNNING_HILDON:
+	HAS_STATUS_ICON = False
 	
-VERSION="3.0"
+VERSION="3.65"
 #DEBUG
 #_USE_KDE_OVERRIDE=False
 # Lucene sucks, forget it
 HAS_LUCENE = False
+##disable while still unstable
+#HAS_XAPIAN = False
+if not HAS_XAPIAN:
+	HAS_SEARCH = False
+	
+if GTK_OK:
+	if HAS_XAPIAN:
+		logging.info("Using Xapian search engine")
+	elif HAS_LUCENE:
+		logging.info("Using Lucene search engine")
+	else:
+		logging.info("xapian or lucene not found, search disabled")
+	
 # Pynotify is still broken, forget it
 HAS_PYNOTIFY = False
 #HAS_PYXML = False
@@ -103,6 +161,13 @@ HAS_PYNOTIFY = False
 #HAS_GNOMEVFS = False
 #HAS_MOZILLA=False
 
+def get_home():
+	if RUNNING_SUGAR:
+		import sugar.env
+		return os.path.join(sugar.env.get_profile_path(), 'penguintv')
+	else:
+		return os.path.join(os.getenv('HOME'), ".penguintv")
+
 def format_size(size):
 	if size > 1073741824:
 		return "%.2f GB" % (float(size)/1073741824)
@@ -114,38 +179,50 @@ def format_size(size):
 		return str(size)+" bytes"
 		
 def GetPrefix():
+	if os.environ.has_key('PTV_PREFIX'):
+		h, t = os.path.split(os.environ['PTV_PREFIX'])
+		return h
 	h, t = os.path.split(os.path.split(os.path.abspath(sys.argv[0]))[0])
 	return h
 	
+_glade_prefix = None
+	
 def get_glade_prefix():
+	global _glade_prefix
+	if _glade_prefix is not None:
+		return _glade_prefix
+
+	logging.debug("finding glade prefix")
+
 	import utils
 	for p in (os.path.join(GetPrefix(),"share","penguintv"),
 		  os.path.join(GetPrefix(),"share"),
 		  os.path.join(os.path.split(os.path.abspath(sys.argv[0]))[0],"share"),
 		  os.path.join(GetPrefix(),"share","sugar","activities","ptv","share"),
-		  os.path.join(os.path.split(os.path.split(utils.__file__)[0])[0],'share')):
+		  os.path.join(os.path.split(os.path.split(utils.__file__)[0])[0],'share'),
+		  "/usr/share/penguintv"):
 		try:
 			os.stat(os.path.join(p,"penguintv.glade"))
-			return p
+			logging.debug("glade prefix found: %s" % (p,))
+			_glade_prefix = p
+			return _glade_prefix
 		except:
 			continue
 	return None
 	
-def get_icon_filename():
-	try:
-		icon_file = GetPrefix()+"/share/pixmaps/penguintvicon.png"
-		os.stat(icon_file)
-	except:
+def get_image_path(filename):
+	for p in (os.path.join(GetPrefix(), 'share', 'pixmaps'),
+			  os.path.join(GetPrefix(), 'share'), #in case the install is still in the source dirs
+			  get_glade_prefix(),
+			  os.path.join(get_glade_prefix(), 'pixmaps')):
 		try:
-			icon_file = GetPrefix()+"/share/penguintvicon.png" #in case the install is still in the source dirs
+			icon_file = os.path.join(p, filename)
 			os.stat(icon_file)
-		except:
-			try:
-				icon_file = get_glade_prefix()+"/penguintvicon.png"
-				os.stat(icon_file)
-			except Exception, e:
-				print "icon not found"
-				raise e
+			return icon_file
+		except Exception, e:
+			continue
+		logging.error("icon not found:" + filename)
+		raise e
 	return icon_file		
 
 def hours(n):  #this func copyright Bram Cohen
@@ -191,11 +268,11 @@ def get_play_command_for(filename):
 			#mimetype = str(kio.KMimeType.findByPath(filename).defaultMimeType())
 			service = kio.KServiceTypeProfile.preferredService(mimetype,"Application")
 			if service is None: #no service, so we use kfmclient and kde should launch a helper window
-				print "unknown type, using kfmclient"
+				logging.info("unknown type, using kfmclient")
 				return "kfmclient exec "
 			full_qual_prog = str(service.exec_()).replace("%U","").strip()
 		except:
-			print "error getting type, using kfmclient"
+			logging.info("error getting type, using kfmclient")
 			return "kfmclient exec "
 	else: #GNOME -- notice how short and sweet this is in comparison :P
 		if HAS_GNOMEVFS:
@@ -203,7 +280,7 @@ def get_play_command_for(filename):
 				mimetype = gnomevfs.get_mime_type(urllib.quote(filename)) #fix for penny arcade filenames
 				full_qual_prog = gnomevfs.mime_get_default_application(mimetype)[2]
 			except:
-				print "unknown type, using gnome-open"
+				logging.info("unknown type, using gnome-open")
 				return "gnome-open "
 		else:
 			# :(
@@ -232,6 +309,7 @@ class GlobDirectoryWalker:
 		self.index = 0
 
 	def __getitem__(self, index):
+		import fnmatch
 		while 1:
 			try:
 				file = self.files[self.index]
@@ -289,13 +367,13 @@ def _mkdir(newdir):
 			os.mkdir(newdir)
 
 
-def my_quote(str):
+def my_quote(s):
 	"""Replaces some problematic characters with html equivalent if necessary"""
 	#right now just & to &amp;, but not &amp; to &amp;amp;
 	#instead of doing this with logic, just "unquote" the amps and then requote them
-	str=string.replace(str,"&amp;","&")
-	str=string.replace(str,"&","&amp;")
-	return str
+	s=string.replace(s,"&amp;","&")
+	s=string.replace(s,"&","&amp;")
+	return s
 	
 def uniquer(seq, idfun=None):
 	if not seq:
@@ -313,32 +391,7 @@ def uniquer(seq, idfun=None):
 		seen[marker] = 1
 		result.append(item)
 	return result
-	
-commands={	'play:':_("Open"),
-			'download:':_("Download"),
-			'downloadqueue:':_("Download And Open"),
-			'pause:':_("Pause"),
-			'cancel:':_("Cancel"),
-			'file://':_("Open File"),
-			'delete:':_("Delete"),
-			'resume:':_("Resume"),
-			'clear:':_("Cancel"),
-			'stop:':_("Stop"),
-			'tryresume:':("Try Resume")	}
-	
-def html_command(command,arg):
-	"""returns something like '<a href="play:%s">Open</a>' for all the commands I have.
-	Dictionary has keys of commands, and returns located strings"""
-	
-	#a couple special cases
-	if command == "redownload":
-		return '<a href="download:'+str(arg)+'">'+_("Re-Download")+"</a>"
 		
-	if command == "retry":
-		return '<a href="download:'+str(arg)+'">'+_("Retry")+'</a>'
-	
-	return '<a href="'+command+str(arg)+'">'+commands[command]+'</a>'
-	
 def is_kde():
 	"""Returns true if the user is running a full KDE desktop, or if it has been overridden for debug
 		purposes"""
@@ -377,7 +430,6 @@ def is_file_media(filename):
 		mimetype = gnomevfs.get_mime_type(urllib.quote(filename))
 	else:
 		return False
-	print mimetype
 	valid_mimes=['video','audio','mp4','realmedia','m4v','mov']
 	for mime in valid_mimes:
 		if mime in mimetype:
@@ -423,27 +475,31 @@ class StrippingParser(HTMLParser.HTMLParser):
             x = ''
         self.result = "%s&%s%s" % (self.result, name, x)
     
-    def handle_starttag(self, tag, attrs):
-        """ Delete all tags except for legal ones """
-        if tag in self.valid_tags:
-            self.result = self.result + '<' + tag
-            for k, v in attrs:
-                if string.lower(k[0:2]) != 'on' and string.lower(v[0:10]) != 'javascript':
-                    self.result = '%s %s="%s"' % (self.result, k, v)
-            endTag = '</%s>' % tag
-            self.endTagList.insert(0,endTag)    
-            self.result = self.result + '>'
+    #def handle_starttag(self, tag, attrs):
+    #    """ Delete all tags except for legal ones """
+    #    if tag in self.valid_tags:
+    #        #self.result = self.result + '<' + tag
+    #        for k, v in attrs:
+    #            if string.lower(k[0:2]) != 'on' and string.lower(v[0:10]) != 'javascript':
+    #                self.result = '%s %s="%s"' % (self.result, k, v)
+    #        #endTag = '</%s>' % tag
+    #        #self.endTagList.insert(0,endTag)    
+    #      	#self.result = self.result + '>'
                 
-    def handle_endtag(self, tag):
-        if tag in self.valid_tags:
-            self.result = "%s</%s>" % (self.result, tag)
-            remTag = '</%s>' % tag
-            self.endTagList.remove(remTag)
+    #def handle_endtag(self, tag):
+    #    if tag in self.valid_tags:
+    #    	
+    #        #self.result = "%s</%s>" % (self.result, tag)
+    #        #remTag = '</%s>' % tag
+    #        #try:
+    #        #	self.endTagList.remove(remTag)
+    #        #except:
+    #        #	pass
 
-    def cleanup(self):
-        """ Append missing closing tags """
-        for j in range(len(self.endTagList)):
-                self.result = self.result + self.endTagList[j]    
+    #def cleanup(self):
+    #    """ Append missing closing tags """
+    #    for j in range(len(self.endTagList)):
+    #            self.result = self.result + self.endTagList[j]    
         
 #usage:
 #def strip(s):
@@ -521,20 +577,24 @@ def init_gtkmozembed():
 	of getting it from the module itself.  good luck with this"""
 
 	assert HAS_MOZILLA
-	os.chdir(os.getenv('HOME'))
-	cmd = "ldd " + gtkmozembed.__file__ + "  | grep xpcom.so"
+	
+	if not os.environ.has_key('MOZILLA_FIVE_HOME'):
+		return False
+	moz_path = os.environ['MOZILLA_FIVE_HOME']
+	cmd = "ldd " + moz_path + '/libxpcom.so  | grep "not found"'
 	p = subprocess.Popen(cmd, shell=True, close_fds=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
 	retval = p.wait()
 	stderr = p.stderr.read()
-	if len(stderr) > 1 or retval != 0:
+	if len(stderr) > 1 or retval == 0:
+		print """***ERROR initializing mozilla.  PenguinTV may crash shortly.  
+You may need to export LD_LIBRARY_PATH=$MOZILLA_FIVE_HOME
+"""
 		return False
-	ldd_output = p.stdout.read()
-	comp_path = os.path.split(ldd_output.split()[2])[0]
-	
+
 	_init_mozilla_proxy()
 	
-	print "initializing mozilla in", comp_path
-	gtkmozembed.set_comp_path(comp_path)
+	logging.info("initializing mozilla in " + str(moz_path))
+	gtkmozembed.set_comp_path(moz_path)
 	
 	return True
 	
@@ -581,13 +641,13 @@ def _init_mozilla_proxy():
 		os.stat(os.path.join(home, "gecko", "prefs.js"))
 		cur_proxy = _get_proxy_prefs(os.path.join(home, "gecko", "prefs.js"))
 		if sys_proxy == cur_proxy:
-			print "gecko proxy settings up to date"
+			logging.info("gecko proxy settings up to date")
 			return
 	except:
 		pass
 
 	try:
-		print "updating gecko proxy settings"
+		logging.info("updating gecko proxy settings")
 		f = open(os.path.join(home, "gecko", "prefs.js"), "w")
 		f.write("""# Mozilla User Preferences
 
@@ -607,7 +667,7 @@ def _init_mozilla_proxy():
 	""" % (sys_proxy['type'], sys_proxy['host'], sys_proxy['port'], sys_proxy['autoconfig_url']))
 		f.close()
 	except:
-		print "WARNING: couldn't create prefs.js, proxy server connections may not work"		
+		logging.warning("couldn't create prefs.js, proxy server connections may not work")
 		
 def _get_proxy_prefs(filename):
 	def isNumber(x):
@@ -626,7 +686,7 @@ def _get_proxy_prefs(filename):
 	try:
 		f = open(filename, "r")
 	except:
-		print "WARNING: couldn't open gecko preferences file ", filename
+		logging.warning("couldn't open gecko preferences file " + filename)
 		return proxy
 	
 	for line in f.readlines():
@@ -649,14 +709,13 @@ def get_pynotify_ok():
 		return False
 
 	# first get what package config reports
-	os.chdir(os.getenv('HOME'))
 	
 	cmd = "pkg-config notify-python --modversion"
 	p = subprocess.Popen(cmd, shell=True, close_fds=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
 	retval = p.wait()
 	stderr = p.stderr.read()
 	if len(stderr) > 1 or retval != 0:
-		print "trouble getting notify-python version from pkg-config, using fallback notifications"
+		logging.warning("trouble getting notify-python version from pkg-config, using fallback notifications")
 		return False
 	major,minor,rev = p.stdout.read().split('.')
 	
@@ -666,10 +725,10 @@ def get_pynotify_ok():
 
 	# if it's bad, return false
 	if minor < 1:
-		print "pynotify too old, using fallback notifications"
+		logging.info("pynotify too old, using fallback notifications")
 		return False
 	if minor == 1 and rev == 0:
-		print "pynotify too old, using fallback notifications"
+		logging.info("pynotify too old, using fallback notifications")
 		return False
 
 	# if it's good, check to see it's not lying about prefix
@@ -678,7 +737,7 @@ def get_pynotify_ok():
 	retval = p.wait()
 	stderr = p.stderr.read()
 	if len(stderr) > 1 or retval != 0:
-		print "trouble getting notify-python prefix from pkg-config, using fallback notifications"
+		logging.info("trouble getting notify-python prefix from pkg-config, using fallback notifications")
 		return False
 	pkgconfig_prefix = p.stdout.read().strip()
 	
@@ -686,7 +745,7 @@ def get_pynotify_ok():
 		dirname = os.path.split(pynotify.__file__)[0]
 		f = open(os.path.join(dirname, "_pynotify.la"))
 	except:
-		print "trouble opening _pynotify.la, using fallback notifications"
+		logging.info("trouble opening _pynotify.la, using fallback notifications")
 		return False
 	
 	libdir_line = ""
@@ -700,15 +759,27 @@ def get_pynotify_ok():
 	libdir_line = libdir_line.strip()
 	
 	if len(libdir_line) == 0:
-		print "trouble reading _pynotify.la, using fallback notifications"
+		logging.info("trouble reading _pynotify.la, using fallback notifications")
 		return False
 	
 	if pkgconfig_prefix not in libdir_line:
-		print "pkgconfig does not agree with _pynotify.la, using fallback notifications"
+		logging.info("pkgconfig does not agree with _pynotify.la, using fallback notifications")
 		return False
 
-	print "using pynotify notifications"
+	logging.info("Using pynotify notifications")
 	return True
+
+def db_except(default_retval=None):
+	def annotate(func):
+		def _exec_cb(self, *args, **kwargs):
+			try:
+				return func(self, *args, **kwargs)
+			except OperationalError:
+				logging.debug("DB Exception")
+				self._handle_db_exception()
+				return _exec_cb(self, *args, **kwargs)
+		return _exec_cb
+	return annotate	
 	
 if is_kde():
 	import kio
